﻿'*************************************************************************
'
' Testprogramm für die Windows-Eigene Spracherkennung
'
' Autor: Joachim Ehrecke / JoE's Softwareschmiede
' Datum: 15.03.2012
' Änderung: 27.01.2013
'
' Copyright: Dieser Programmkode ist entsprechend deutscher 
' Rechtssprechung mit der Entstehung des Werks urheberrechtlich geschützt
' und darf nur mit schriftlicher Erlaubnis des Autors wiedergegeben, 
' vervielfältigt, abgedruckt, veröffentlicht, in eigene oder fremde Soft-
' ware oder Programme integriert oder auf andere Weise weiterverwendet 
' werden. 
'
' Dieses Programm dient nur der Vermittlung von Grundlagenwissen über
' die Benutzung der betriebssystemeigenen Microsoft Speech Recognition
' Engine und erhebt keinen Anspruch auf Vollständigkeit und 
' Fehlerfreiheit.
'
'
'*************************************************************************


Option Explicit On
Option Strict On


#Region "Imports"

Imports System.Globalization
Imports System.Threading
Imports System.Text
Imports System.Collections.Generic
Imports System.Speech
Imports System.Xml
Imports System.Xml.XPath
Imports System.Windows.Forms.VisualStyles.VisualStyleElement
Imports System.ComponentModel

#End Region



Public Class FormiNetSpeechRec


#Region "Declarations"

    'Die Spracherkennungs-Maschine initialisieren
    'Private WithEvents SpeechRecEngine As New MSSpeechRecognition(Me)
    Private WithEvents SpeechRecEngine As MSSpeechRecognition
    'Grammatik Dateinamen festlegen
    Private FileName As String = "grammar.de-DE.xml"
    'Einen FIFO für die angekommenden und erkannten Befehle anlegen
    Private Commands As New Queue(Of String)

#End Region



#Region "Formular"

    ''' <summary>
    ''' Form: Neu anliegen einer Instanz der Form
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        With My.Settings
            Try
                'Ist der Globalisation-String leer?
                If String.IsNullOrEmpty(.Localization) Then
                    'Wenn die Sprache nicht definiert wurde, dann die Betriebssystem-Sprache wählen
                    .Localization = My.Computer.Info.InstalledUICulture.Name
                End If
                'Anzeige der ausgwählten Sprache. Dieses ist eigentlich nur zum Testen gedacht,
                ' da die Sprachumschaltung automatisch passiert
                Threading.Thread.CurrentThread.CurrentCulture = New Globalization.CultureInfo(.Localization, True)
                Threading.Thread.CurrentThread.CurrentUICulture = New Globalization.CultureInfo(.Localization, True)
                My.Application.ChangeCulture(.Localization)
                My.Application.ChangeUICulture(.Localization)

            Catch exc As Exception
                'Die Sprache war wohl fehlerhaft, also den fehlerhaften String korrigieren
                .Localization = My.Computer.Info.InstalledUICulture.Name
                .Save()
            End Try
        End With

        ' Die Spracherkennungs-Maschine initialisieren
        SpeechRecEngine = New MSSpeechRecognition(Me)

        'Dieser Aufruf ist für den Designer erforderlich.
        InitializeComponent()
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FormiNetSpeechRec_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        'Lokale Variablen
        Dim ThisForm As Form = CType(sender, Form)

        'In der Titelzeile den Namen und die Versionsnummer der Applikation anzeigen
        ThisForm.Text = My.Application.Info.Title.ToString & " V" & My.Application.Info.Version.ToString

        With My.Settings
            'Position und Größe wieder mit den gespeicherten Werten herstellen
            ' Position
            ThisForm.Location = New Point(.Left, .Top)
            ' Größe
            ThisForm.Height = .Height
            ThisForm.Width = .Width
        End With

        'Initialisieren der Windows Speech Recognition
        With SpeechRecEngine
            ' Grammatik laden
            .GrammarFileName = "grammar.de-DE.xml"
            ' Ein- oder Ausschalten der Spracherkennung
            .On = My.Settings.OnFlag
            ' Abfragen, ob die Spracherkennung auf diesem Rechner möglich ist (Mikrofon ist notwendig)
            Dim ChassisNo As Integer = 0
            Dim ChassisName As String = String.Empty
            Dim SpeechRecPossible As Boolean = False
            Call .ComputerType(ChassisNo, ChassisName, SpeechRecPossible)
            ToolStripStatusLabelError.Text = String.Format(ToolStripStatusLabelError.Text, ChassisName, SpeechRecPossible.ToString)
            'Anzeigen der Standardwerte
            ' Status
            LabelRecAudioState.Text = .Engine.AudioState.ToString
            ' Erkennungssicherheit
            TrackBarConfidence.Value = My.Settings.Confidence
        End With
    End Sub

    ''' <summary>
    ''' Form wird geschlossen
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FormiSpeechRec_FormClosing(sender As Object, e As FormClosingEventArgs) Handles Me.FormClosing
        'Lokale Variablen
        Dim ThisForm As Form = CType(sender, Form)

        With My.Settings
            'Veränderte Größe speichern
            .Height = ThisForm.Height
            .Width = ThisForm.Width
        End With
    End Sub

    ''' <summary>
    ''' Die Position der Form hat sich geändert
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FormiSpeechRec_LocationChanged(sender As Object, e As EventArgs) Handles Me.LocationChanged
        'Lokale Variablen
        Dim ThisForm As Form = CType(sender, Form)

        With My.Settings
            'Position speichern
            .Top = ThisForm.Location.Y
            .Left = ThisForm.Location.X
        End With
    End Sub

    ''' <summary>
    ''' Ein- und Ausschalten der SR-Engine
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ButtonSREngineOnOff_Click(sender As Object, e As EventArgs) Handles ButtonSREngineOnOff.Click
        'Sprach-Maschine ein- oder ausschalten
        With SpeechRecEngine
            My.Settings.OnFlag = Not My.Settings.OnFlag
            .On = My.Settings.OnFlag
        End With
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub CheckBoxDictation_CheckedChanged(sender As Object, e As EventArgs) Handles CheckBoxDictation.CheckedChanged
        'Lokale Variablen
        Dim MyCheckBox As CheckBox = CType(sender, CheckBox)

        'Diktat-Modus ein- oder ausschalten
        SpeechRecEngine.Dictation = MyCheckBox.Checked
    End Sub

    ''' <summary>
    ''' Confidence: 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TrackBarConfidence_Scroll(sender As Object, e As EventArgs) Handles TrackBarConfidence.Scroll
        'Lokale Variablen
        Dim ThisTrackBar As System.Windows.Forms.TrackBar = CType(sender, System.Windows.Forms.TrackBar)

        'TrackBar-Wert korrigieren und übernehmen
        SpeechRecEngine.ConfidenceThreshold = ThisTrackBar.Value
    End Sub

    ''' <summary>
    ''' Confidence: 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TrackBarConfidence_ValueChanged(sender As Object, e As EventArgs) Handles TrackBarConfidence.ValueChanged
        'Die Sicherheit entsprechend TrackBarConfidence_Scroll einstellen
        Call TrackBarConfidence_Scroll(sender, e)
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub CheckBoxBeep_CheckedChanged(sender As Object, e As EventArgs) Handles CheckBoxBeep.CheckedChanged
        'Lokale Variablen
        Dim ThisCheckBox As CheckBox = CType(sender, CheckBox)

        'Den BEEP ein- und ausschalten, wenn die Engine schon initialisiert wurde
        SpeechRecEngine.BeepOn = ThisCheckBox.Checked
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ButtonUIChangeLanguage_Click(sender As Object, e As EventArgs) Handles ButtonUIChangeLanguageDE.Click
        'Sprache auf Deutsch umstellen
        Call ChangeLanguage("de-DE")
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ButtonUIChangeLanguageGB_Click(sender As Object, e As EventArgs) Handles ButtonUIChangeLanguageGB.Click
        'Sprache auf Deutsch umstellen
        Call ChangeLanguage("en-GB")
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ButtonCallSRWindows_Click(sender As System.Object, e As System.EventArgs) Handles ButtonCallSRWindows.Click
        'Lokale Variablen
        Dim Retval As Boolean = True                'Rückgabewert für OK-Meldung False = Fehler
        Dim CPLProcess As New Process               'Prozess initialisieren...

        Try
            'Prozess vorbereiten
            With CPLProcess.StartInfo
                ' Fehlerdialog verwenden
                .ErrorDialog = False
                'Arbeits-Verzeichnis angeben 
                .WorkingDirectory = Environment.ExpandEnvironmentVariables("C:\Windows\SysWOW64\")
                ' Aufrufende ausfuehrende Datei an FileName uebergeben
                .FileName = "control.exe"
                'Argument für den aufzurufenden Prozess
                .Arguments = "/name Microsoft.SpeechRecognition"
                'Fenster-Start-Style initialisieren
                .WindowStyle = ProcessWindowStyle.Normal
            End With

            'Prozess starten
            CPLProcess.Start()

            'Fehlerauswertung
        Catch Exc As Exception
            'Hier mal nichts machen
            Debug.Print("ButtonCallSRWindows_Click: " & Exc.Message)
        End Try
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ButtonClear_Click(sender As System.Object, e As System.EventArgs) Handles ButtonClear.Click
        'Löschen des Textes
        TextBoxRecognizerResult.Text = String.Empty
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TimerCommands_Tick(sender As System.Object, e As System.EventArgs) Handles TimerCommands.Tick
        'Lokale Variablen
        Dim Command As String = String.Empty
        Dim OldCursor As Cursor = Cursor

        Try
            'Maus als Eieruhr darstellen
            Cursor = Cursors.WaitCursor

            'Timer abschalten, da er ja nur einmal ausgelöste werden sollte
            TimerCommands.Enabled = False

            'Befehlsbearbeitungsschleife: Alle Befehle im Fifo bearbeiten
            Do While Commands.Count > 0
                'Befehle abarbeiten
                Command = Commands.Dequeue()

                'Befehle herausfinden und dann die entsprechende Funktione aufrufen
                Select Case Command
                    Case "Aus", "Off"                               'Spracherkennung ausschalten
                        ButtonSREngineOnOff_Click(ButtonSREngineOnOff, New EventArgs)

                    Case "Spracherkennung von Windows öffnen"       'Spracherkennung von Windows öffnen
                        Call ButtonCallSRWindows_Click(ButtonCallSRWindows, New EventArgs)

                    Case "Aufzeichnung löschen"                     'Befehl Aufzeichnung löschen
                        Call ButtonClear_Click(ButtonClear, New EventArgs)

                        'Case "Beenden", "Close"                         'Programm beenden
                        '    'Threading.Thread.Sleep(1000)
                        '    'End

                    Case "Klingelton", "Beep"                       'Klingelton umschalten (An/Aus)
                        CheckBoxBeep.Checked = Not CheckBoxBeep.Checked

                    Case "Blau", "Blue"                             'Farbe Blau einschalten
                        PictureBoxColor.BackColor = Color.LightBlue

                    Case "Grün", "Green"                            'Farbe Grün einschalten
                        PictureBoxColor.BackColor = Color.LightGreen

                    Case "Gelb", "Yellow"                           'Farbe Grün einschalten
                        PictureBoxColor.BackColor = Color.Yellow

                    Case "Standardfarbe", "Standard Colour"         'Standard-Farbe einschalten
                        PictureBoxColor.BackColor = Color.FromKnownColor(KnownColor.Control)

                    Case Else       'Den erkannten Befehl gibt es nicht in dieser "Select Case"-Listung
                        'Benachrichtigung über den Befhle anzeigen
                        TextBoxRecognizerResult.Text = Command.ToString & vbCrLf & TextBoxRecognizerResult.Text
                End Select
            Loop

            'Fehlerauswertung
        Catch Exc As Exception
            'Fehler ausgeben
            Debug.Print("TimerCommands_Tick: " & Exc.Message)
        Finally
            'Maus wieder als Standard anzeigen
            Cursor = OldCursor
        End Try
    End Sub

#End Region



#Region "Speech Recognition Events"

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_Error(sender As Object, e As MSSpeechRecognition.SRErrorEventArgs) Handles SpeechRecEngine.Error
        'Fehlermeldung formatiert ausgeben
        ToolStripStatusLabelError.Text = e.ErrorDate.ToString & " / " & e.Source & " / " & e.Exc.Message
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_AudioLevelUpdated(sender As Object, e As System.Speech.Recognition.AudioLevelUpdatedEventArgs) Handles SpeechRecEngine.SR_AudioLevelUpdated
        'Audio-Level anzeigen
        ProgressBarRecAudioLevel.Value = e.AudioLevel

        'Den Hintergrund entsprechend zum Level setzen
        PictureBoxMicro.BackColor = SetColorFromLevel(e.AudioLevel)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_AudioSignalProblemOccurred(sender As Object, e As System.Speech.Recognition.AudioSignalProblemOccurredEventArgs) Handles SpeechRecEngine.SR_AudioSignalProblemOccurred
        'Ergebnis der Erkennung Anzeigen
        LabelSRAudioProblems.Text = e.AudioSignalProblem.ToString
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_AudioStateChanged(sender As Object, e As System.Speech.Recognition.AudioStateChangedEventArgs) Handles SpeechRecEngine.SR_AudioStateChanged
        'Status anzeigen
        LabelRecAudioState.Text = e.AudioState.ToString

        ''Da die Einstellung der Erkennungsschwelle nur bei nicht erkennender Sprach-Engine
        '' funktioniert, wird die Veränderung des Confidence-Wertes nur bei gestoppter
        '' Engine zugelassen -> Trackbar für die Änderung abschalten wenn Engine in 
        '' Betrieb ist
        'If e.AudioState = Recognition.AudioState.Stopped Then
        '    'Einschalten
        '    TrackBarConfidence.Enabled = True
        'Else
        '    'Ausschalten
        '    TrackBarConfidence.Enabled = False
        'End If
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SR_ConfidenceChanged(sender As Object, e As MSSpeechRecognition.SRConfidenceEventArgs) Handles SpeechRecEngine.SR_ConfidenceChanged
        'Anzeigen der Sicherheit der Spracherkennung
        LabelConfidenceValue.Text = String.Format("{0} %", SpeechRecEngine.ConfidenceThreshold)

        'Speichern des Sicherheits-Wertes in den Einstellungen
        My.Settings.Confidence = SpeechRecEngine.ConfidenceThreshold
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_EmulateRecognizeCompleted(sender As Object, e As System.Speech.Recognition.EmulateRecognizeCompletedEventArgs) Handles SpeechRecEngine.SR_EmulateRecognizeCompleted
        'Ergebnis Anzeigen
        TextBoxRecognizerResult.Text = "Speech ERC: " & e.Result.ToString & " / " & e.UserState.ToString
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SR_EngineOn(sender As Object, e As MSSpeechRecognition.SROnEventArgs) Handles SpeechRecEngine.SR_EngineOn
        'Lokale Variablen
        Dim MySpeechEngine As MSSpeechRecognition = CType(sender, MSSpeechRecognition)
        Dim MyInfoSB As New StringBuilder(String.Empty)

        With ButtonSREngineOnOff
            'Ist die Engine eingeschaltet?
            If e.On Then
                'Die eingeschaltete Speech-Recognition durch Farbe und Text anzeigen
                .BackColor = Color.FromArgb(80, Color.Green)

                'Zusätzlich Infos zur Engine anzeigen
                With MySpeechEngine.Engine
                    'Programminformationen
                    MyInfoSB.AppendLine("Program Info:")
                    MyInfoSB.AppendLine(vbTab & "Product Name: " & My.Application.Info.ProductName)
                    MyInfoSB.AppendLine(vbTab & "Version: " & My.Application.Info.Version.ToString)
                    MyInfoSB.AppendLine(vbTab & "Author: " & My.Application.Info.Copyright)
                    'Leerzeile einfügen
                    MyInfoSB.AppendLine()
                    'Informationen zur Spracherkennung anzeigen
                    MyInfoSB.AppendLine("Current Recognizer:")
                    MyInfoSB.AppendLine(vbTab & "Description: " & .RecognizerInfo.Description.ToString)
                    MyInfoSB.AppendLine(vbTab & "Culture: " & .RecognizerInfo.Culture.ToString)
                    MyInfoSB.AppendLine(vbTab & "Name: " & .RecognizerInfo.Name.ToString)
                    MyInfoSB.AppendLine(vbTab & "ID: " & .RecognizerInfo.Id.ToString)
                    'Leerzeile einfügen
                    MyInfoSB.AppendLine()
                    'Die Audio-Informationen anzeigen
                    MyInfoSB.AppendLine("Audio Formats: ")
                    For Each AudioInfo As AudioFormat.SpeechAudioFormatInfo In .RecognizerInfo.SupportedAudioFormats
                        With AudioInfo
                            MyInfoSB.AppendLine(vbTab & .EncodingFormat.ToString & " / " & .AverageBytesPerSecond.ToString & _
                               " / " & .BitsPerSample.ToString & .BlockAlign.ToString & " / " & .ChannelCount & " / " & _
                               .SamplesPerSecond.ToString)
                        End With
                        Application.DoEvents()
                    Next
                    'Leerzeile einfügen
                    MyInfoSB.AppendLine()
                    'Weitere Informationen abrufen
                    MyInfoSB.AppendLine("Add. Infos: ")
                    Dim AddInfo As System.Collections.Generic.IDictionary(Of String, String) = .RecognizerInfo.AdditionalInfo
                    With AddInfo
                        For iCnt As Integer = 0 To AddInfo.Count - 1
                            MyInfoSB.AppendLine(vbTab & .ElementAt(iCnt).Key.ToString & ": " & .ElementAt(iCnt).Value.ToString)
                        Next
                    End With
                    'Leerzeile einfügen
                    MyInfoSB.AppendLine()
                    'Installierte Sprach-Erkennungs-Maschinen anzeigen
                    MyInfoSB.AppendLine("Installed Recognizers: ")
                    For Each RecInfo As Recognition.RecognizerInfo In Recognition.SpeechRecognitionEngine.InstalledRecognizers
                        'Beschreibung anzeigen 
                        MyInfoSB.AppendLine(vbTab & RecInfo.Description)
                    Next
                    'Informationen in der TextBox ausgeben
                    TextBoxSRInfo.Text = MyInfoSB.ToString
                End With
            Else
                'Die ausgeschaltete Speech-Recognition durch Farbe und Text anzeigen
                .BackColor = Color.FromArgb(80, Color.Red)
            End If
        End With
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_LoadGrammarCompleted(sender As Object, e As System.Speech.Recognition.LoadGrammarCompletedEventArgs) Handles SpeechRecEngine.SR_LoadGrammarCompleted
        'Lokale Variablen
        Dim MyInfoSB As New StringBuilder(String.Empty)

        'Befehle in der Listbox anzeigen
        ListBoxGrammar.Items.AddRange(SpeechRecEngine.ListOfCommands)
        LabelCommandsDescription.Text = String.Format(LabelCommandsDescription.Text, SpeechRecEngine.ListOfCommands.Count.ToString)

        'Info zur geladenen Grammatik anzeigen
        If e.Error Is Nothing Then
            MyInfoSB.AppendLine(String.Format("Grammar: Grammar Load Completed ({0})", e.Grammar.Name))
        Else
            'Fehlermeldung ausgeben
            MyInfoSB.AppendLine(String.Format("Grammar: Failed With {0}", e.Error.GetType().Name))
        End If
        'Informationen in der TextBox ausgeben
        TextBoxRecognizerResult.Text = MyInfoSB.ToString
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_RecognizeCompleted(sender As Object, e As System.Speech.Recognition.RecognizeCompletedEventArgs) Handles SpeechRecEngine.SR_RecognizeCompleted
        'Ergebnis der Erkennung Anzeigen
        If e.Cancelled Then
            'Erkennung abgebrochen (kann man melden, muss man aber nicht)
            TextBoxRecognizerResult.Text = "Cancelled: " & TextBoxRecognizerResult.Text
        Else
            'Es wurde ein Befehl erkannt
            TextBoxRecognizerResult.Text = "Beendet: " & e.Result.Text & " / Sicherheit: " & (e.Result.Confidence * 100).ToString("##") & "%" & _
                    vbCrLf & TextBoxRecognizerResult.Text
        End If
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_RecognizerUpdateReached(sender As Object, e As System.Speech.Recognition.RecognizerUpdateReachedEventArgs) Handles SpeechRecEngine.SR_RecognizerUpdateReached
        'Ergebnis Anzeigen
        TextBoxRecognizerResult.Text = "Speech RUR: " & e.AudioPosition.ToString & " / " & e.UserToken.ToString
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SpeechDetected(sender As Object, e As System.Speech.Recognition.SpeechDetectedEventArgs) Handles SpeechRecEngine.SR_SpeechDetected
        'Ergebnis der Erkennung Anzeigen
        TextBoxRecognizerResult.Text = "Sprech-Zeit: " & e.AudioPosition.ToString & vbCrLf & TextBoxRecognizerResult.Text
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SpeechHypothesized(sender As Object, e As System.Speech.Recognition.SpeechHypothesizedEventArgs) Handles SpeechRecEngine.SR_SpeechHypothesized
        'Ergebnis der Erkennung Anzeigen
        TextBoxRecognizerResult.Text = "Vermutung: " & e.Result.Text & " / Sicherheit: " & (e.Result.Confidence * 100.0).ToString("##") & "%" & _
                vbCrLf & TextBoxRecognizerResult.Text
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SpeechRecognitionRejected(sender As Object, e As System.Speech.Recognition.SpeechRecognitionRejectedEventArgs) Handles SpeechRecEngine.SR_SpeechRecognitionRejected
        'Ergebnis der Erkennung Anzeigen
        TextBoxRecognizerResult.Text = "Verworfen: " & e.Result.Text & " / Sicherheit: " & (e.Result.Confidence * 100.0).ToString("##") & "%" & _
                vbCrLf & TextBoxRecognizerResult.Text

        'Den Hintergrund entsprechend zur Erkennung setzen
        PictureBoxRecognition.BackColor = System.Drawing.Color.Red
        TimerRecognitionFlag.Enabled = True
    End Sub

    ''' <summary>
    '''  Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SpeechRecognized(sender As Object, e As System.Speech.Recognition.SpeechRecognizedEventArgs) Handles SpeechRecEngine.SR_SpeechRecognized
        'Ist das Diktat eingeschaltet? Wenn ja nur noch den Text anzeigen
        If SpeechRecEngine.Dictation Then
            RichTextBoxDictation.Text &= e.Result.Text
        Else
            'Ergebnis der Erkennung Anzeigen
            TextBoxRecognizerResult.Text = "Erkannt: " & e.Result.Text & " / Sicherheit: " & (e.Result.Confidence * 100).ToString("##") & "%" & _
                    vbCrLf & TextBoxRecognizerResult.Text

            'Das erkannte Kommando (oder Text) in den FIFO ablegen
            Commands.Enqueue(e.Result.Text)
            'Auswertungstimer starten. Damit wird ein quasi Multitasking erreicht
            TimerCommands.Enabled = True

            'Den Hintergrund entsprechend zur Erkennung setzen
            PictureBoxRecognition.BackColor = System.Drawing.Color.Green
            TimerRecognitionFlag.Enabled = True

            'Den erkannten Befehl in der Listbox selektieren
            ListBoxGrammar.SelectedIndex = ListBoxGrammar.FindStringExact(e.Result.Text)
        End If
    End Sub

#End Region



#Region "Dictation"

    ''' <summary>
    ''' Diktat: 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub RichTextBoxDictation_TextChanged(sender As Object, e As EventArgs) Handles RichTextBoxDictation.TextChanged
        'Lokale Variablen
        Dim MyRichTextBox As System.Windows.Forms.RichTextBox = CType(sender, System.Windows.Forms.RichTextBox)

        With MyRichTextBox

            'Cursor an das Ende des Textes setzen
            .SelectionStart = .Text.Length
            .ScrollToCaret()
        End With
    End Sub

    ''' <summary>
    ''' Diktat: 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ButtonDictateClearText_Click(sender As Object, e As EventArgs) Handles ButtonDictateClearText.Click
        'Den Text des Diktats löschen
        RichTextBoxDictation.Clear()
    End Sub

    ''' <summary>
    ''' Diktat:  Der Text für das Starten des Diktats wurde geändert
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TextBoxDictPrecedingText_TextChanged(sender As Object, e As EventArgs) Handles TextBoxDictPrecedingText.TextChanged
        'Lokale Variablen
        Dim MyTextBox As System.Windows.Forms.TextBox = CType(sender, System.Windows.Forms.TextBox)

        'Den text als "Start Diktat" benutzen
        SpeechRecEngine.DictPrecedingText = MyTextBox.Text.Trim
    End Sub

    ''' <summary>
    ''' Diktat: Der Text für das Beenden des Diktats wurde geändert
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TextBoxDictSubsequentText_TextChanged(sender As Object, e As EventArgs) Handles TextBoxDictSubsequentText.TextChanged
        'Lokale Variablen
        Dim MyTextBox As System.Windows.Forms.TextBox = CType(sender, System.Windows.Forms.TextBox)

        'Den Text als "Beende Diktat" benutzen
        SpeechRecEngine.DictSubsequentText = MyTextBox.Text.Trim
    End Sub

    ''' <summary>
    ''' Diktat: Das Senden des Textes an andere Applikationen ein- oder ausschalten
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub CheckBoxDictSendText_CheckedChanged(sender As Object, e As EventArgs) Handles CheckBoxDictSendText.CheckedChanged
        'Lokale Variablen
        Dim MyCheckBox As CheckBox = CType(sender, CheckBox)

        SpeechRecEngine.DictationSendText = MyCheckBox.Checked
    End Sub

#End Region



#Region "Verschiedene Subs und Functions"

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="AudioLevel"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function SetColorFromLevel(AudioLevel As Integer) As System.Drawing.Color
        'Lokale Variablen
        Dim IntLevel As Single = 0.0
        Dim RetValColor As System.Drawing.Color

        'Den Audiolevel prüfen und gegebenenfalls korrigieren
        If AudioLevel > 100 Then AudioLevel = 100
        If AudioLevel < 0 Then AudioLevel = 0

        'Den Level verstärken wenn er sehr klein ist
        IntLevel = CSng(Math.Pow(CDbl(AudioLevel / 100), CDbl(1 / 2)) * 255)

        'Farbe setzen
        'BGColor = System.Drawing.Color.FromArgb(80, 255 - CInt(IntLevel * 255), 255 - CInt(IntLevel * 255), 255 - CInt(IntLevel * 255))
        RetValColor = System.Drawing.Color.FromArgb(90, 255 - CInt(IntLevel), 255 - CInt(IntLevel), 255)

        'Berechnete Farbe zurückgeben
        Return RetValColor
    End Function

#End Region



#Region "Timer"

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TimerRecognitionFlag_Tick(sender As Object, e As EventArgs) Handles TimerRecognitionFlag.Tick
        'Farbe des Controls für die Erkennungsanzeige wieder auf Standard setzen
        PictureBoxRecognition.BackColor = Color.FromKnownColor(KnownColor.Control)
    End Sub

#End Region




#Region "Language"

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="LocalizationString"></param>
    ''' <remarks></remarks>
    Private Sub ChangeLanguage(ByVal LocalizationString As String)
        'Den Lokalisations-Text (z.B. "de-DE", "en-GB") in den Einstellungen schreiben
        My.Settings.Localization = LocalizationString
        'Einstellungen speichern
        My.Settings.Save()

        'Programm neu starten
        Application.Restart()
    End Sub

    Private Sub LabelRecAudioState_Click(sender As Object, e As EventArgs) Handles LabelRecAudioState.Click

    End Sub

#End Region



End Class
