﻿'*************************************************************************
'
' Testprogramm für die Windows-Eigene Spracherkennung
'
' Autor: Joachim Ehrecke / JoE's Softwareschmiede
' Datum: 15.03.2012
' Änderung: 05.08.2014
'
' Copyright: Dieser Programmkode ist entsprechend deutscher 
' Rechtssprechung mit der Entstehung des Werks urheberrechtlich geschützt
' und darf nur mit schriftlicher Erlaubnis des Autors wiedergegeben, 
' vervielfältigt, abgedruckt, veröffentlicht, in eigene oder fremde Soft-
' ware oder Programme integriert oder auf andere Weise weiterverwendet 
' werden. 
'
' Dieses Programm dient nur der Vermittlung von Grundlagenwissen über
' die Benutzung der betriebssystemeigenen Microsoft Speech Recognition
' Engine und erhebt keinen Anspruch auf Vollständigkeit und 
' Fehlerfreiheit.
'
'
'*************************************************************************


Option Explicit On
Option Strict On


#Region "Imports"

Imports System.Collections.Generic
Imports System.Speech
Imports System.Speech.Recognition
Imports System.Xml
Imports System.Xml.XPath
Imports System.Management

#End Region



Public Class MSSpeechRecognition


#Region "Declarations"

    'Die Eltern-Form
    Private MyForm As Form

    'Liste der vorhandenen Befehle
    Public ListOfCommands() As String
    'Die Spracherkennungs-Maschine initialisieren
    Public WithEvents Engine As SpeechRecognitionEngine

    'Timer für das Entkoppeln von Oberfläche und der Confidence Einstellung
    Private WithEvents TimerDecoupleConfidence As Timer
    'Grammatik der SpeechRecognition
    Private MyGrammar As Grammar
    'Grammatik Dateinamen festlegen
    Private _GrammarFileName As String = String.Empty
    'Erkennungs-Level
    Private _ConfidenceThreshold As Integer = 20
    'Diktat-Modus
    Private _Dictation As Boolean
    'Grammatik für Diktat erzeugen
    Private MyDictationGrammar As DictationGrammar
    'Diktat Start
    Private _DictPrecedingText As String = "Start"
    'Diktat Stop
    Private _DictSubsequentText As String = "Stop"
    'Diktat senden des Texts an andere Applikationen
    Private _DictationSendText As Boolean = False
    'Speech-Recognition-Engine eingeschaltet
    Private _On As Boolean = False
    'Beep-Flag, gibt an ob Beep ausgelöst werden soll oder nicht
    Private _BeepOn As Boolean = False
    'Flag ob die Initialisierung erfolgreich war
    Private intInitFlag As Boolean = False

#End Region



#Region "Class"

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New(Formular As Form)
        'Daten übernehmen
        Me.MyForm = Formular

        'Initialisieren der Standardwerte
        Call StdInit(TimerDecoupleConfidence)

        'SpeechEngine Initialisierung erfolgreich abgeschlossen, oder?
        intInitFlag = SpeechRecEngineInit(Engine, MyGrammar, _GrammarFileName, _Dictation, MyDictationGrammar)
        'Weitere Initialisierungen wenn die SpeechEngine vorhanden ist
        If intInitFlag Then
            'Grammatik laden
            ListOfCommands = SpeechRecLoadGrammar(Engine, MyGrammar, _GrammarFileName, _Dictation,
                        MyDictationGrammar, _DictPrecedingText, _DictSubsequentText)

            'Setzen der Erkennungsschwelle auf 20%
            ConfidenceThreshold = 20
        End If
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="GrammarFileName"></param>
    ''' <remarks></remarks>
    Public Sub New(Formular As Form, GrammarFileName As String)
        'Daten übernehmen
        ' Eltern-Formular
        Me.MyForm = Formular
        ' Grammatik-Datei-Namen speichern
        _GrammarFileName = GrammarFileName

        'Initialisieren der Standardwerte
        Call StdInit(TimerDecoupleConfidence)

        'SpeechEngine Initialisierung erfolgreich abgeschlossen, oder?
        intInitFlag = SpeechRecEngineInit(Engine, MyGrammar, _GrammarFileName, _Dictation, MyDictationGrammar)

        'Setzen der Erkennungsschwelle auf 40%
        If intInitFlag Then ConfidenceThreshold = 40
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overrides Sub Finalize()
        'Beenden der Spracherkennung
        'If Engine IsNot Nothing Then Engine.Dispose()
        If Engine IsNot Nothing Then SpeechRecEngineExit(Engine)

        MyBase.Finalize()
    End Sub

#End Region



#Region "Class Events"

    'Event zur Anzeige für neue Meldungen
    Public Event [Error](sender As Object, e As SRErrorEventArgs)

    ''' <summary>
    ''' Event Argument zur Message-Übermittlung
    ''' </summary>
    ''' <remarks></remarks>
    Public Class SRErrorEventArgs
        'Soll von den Standard EventArgs abstammen und erben
        Inherits EventArgs

        'Daten
        Public ErrorDate As Date
        Public Source As String
        Public Exc As Exception

        Public Sub New(ErrorDate As Date, Source As String, Exc As Exception)
            'Daten übernehmen
            Me.ErrorDate = ErrorDate
            Me.Source = Source
            Me.Exc = Exc
        End Sub
    End Class

    'Event zur Anzeige für neue Meldungen
    Public Event SR_EngineOn(sender As Object, e As SROnEventArgs)

    ''' <summary>
    ''' Event Argument zur Message-Übermittlung
    ''' </summary>
    ''' <remarks></remarks>
    Public Class SROnEventArgs
        'Soll von den Standard EventArgs abstammen und erben
        Inherits EventArgs

        'Daten
        Public [On] As Boolean

        Public Sub New([On] As Boolean)
            'Daten übernehmen
            Me.On = [On]
        End Sub
    End Class

    'Event zur Anzeige für neue Meldungen
    Public Event SR_ConfidenceChanged(sender As Object, e As SRConfidenceEventArgs)

    ''' <summary>
    ''' Event Argument zur Message-Übermittlung
    ''' </summary>
    ''' <remarks></remarks>
    Public Class SRConfidenceEventArgs
        'Soll von den Standard EventArgs abstammen und erben
        Inherits EventArgs

        'Daten
        Public Confidence As Integer

        Public Sub New(Confidence As Integer)
            'Daten übernehmen
            Me.Confidence = Confidence
        End Sub
    End Class

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_AudioLevelUpdated(sender As Object, e As System.Speech.Recognition.AudioLevelUpdatedEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_AudioSignalProblemOccurred(sender As Object, e As System.Speech.Recognition.AudioSignalProblemOccurredEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_AudioStateChanged(sender As Object, e As System.Speech.Recognition.AudioStateChangedEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_EmulateRecognizeCompleted(sender As Object, e As System.Speech.Recognition.EmulateRecognizeCompletedEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_LoadGrammarCompleted(sender As Object, e As System.Speech.Recognition.LoadGrammarCompletedEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_RecognizeCompleted(sender As Object, e As System.Speech.Recognition.RecognizeCompletedEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_RecognizerUpdateReached(sender As Object, e As System.Speech.Recognition.RecognizerUpdateReachedEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_SpeechDetected(sender As Object, e As System.Speech.Recognition.SpeechDetectedEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_SpeechHypothesized(sender As Object, e As System.Speech.Recognition.SpeechHypothesizedEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_SpeechRecognitionRejected(sender As Object, e As System.Speech.Recognition.SpeechRecognitionRejectedEventArgs)

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event SR_SpeechRecognized(sender As Object, e As System.Speech.Recognition.SpeechRecognizedEventArgs)

#End Region




#Region "Properties"

    ''' <summary>
    ''' ON: Ein- oder ausschalten der Speech-Recognition-Engine
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property [On]() As Boolean
        Get
            'Rückgabe des Einschaltwerts der Engine
            Return _On
        End Get
        Set(value As Boolean)
            'Erkennung starten oder stoppen
            _On = EngineOnOff(Engine, value)
        End Set
    End Property

    ''' <summary>
    ''' Beep ein- oder Ausschalten
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property BeepOn As Boolean
        Get
            'Aktuellen Zustand des Beep-Flags zurückgeben
            Return _BeepOn
        End Get
        Set(value As Boolean)
            'Schreiben des Beep-Flags
            _BeepOn = value
        End Set
    End Property

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property GrammarFileName As String
        Get
            'Rückgabe des Dateinamens der Grammatikdatei
            Return _GrammarFileName
        End Get
        Set(value As String)
            'Grammatikdatei-Namen setzen
            _GrammarFileName = value
            'Grammatikdatei laden
            ListOfCommands = SpeechRecLoadGrammar(Engine, MyGrammar, _GrammarFileName, _Dictation, MyDictationGrammar,
                        _DictPrecedingText, _DictSubsequentText)
        End Set
    End Property

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property ConfidenceThreshold As Integer
        Get
            'Den momentanen Level zurückgeben
            'intConfidenceLevel = CInt(Engine.QueryRecognizerSetting("CFGConfidenceRejectionThreshold"))
            Return _ConfidenceThreshold
        End Get
        Set(value As Integer)
            'Setzen des neuen Confidence-Werts
            _ConfidenceThreshold = value

            'Timer zurücksetzen und neu starten
            TimerDecoupleConfidence.Enabled = True
            TimerDecoupleConfidence.Start()
        End Set
    End Property

    ''' <summary>
    ''' Die Spracherkennung in den Diktat-Modus setzen
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Dictation As Boolean
        Get
            'Zurückgeben des Flags
            Return _Dictation
        End Get
        Set(value As Boolean)
            'Setzen des Flags
            _Dictation = value

            'Diktat-Modus ein- oder ausschalten
            ListOfCommands = SpeechRecLoadGrammar(Engine, MyGrammar, _GrammarFileName, _Dictation, MyDictationGrammar,
                        _DictPrecedingText, _DictSubsequentText)
        End Set
    End Property

    ''' <summary>
    ''' Diktat: Schreiben oder Lesen des Werts für das Starten des Diktats
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property DictPrecedingText As String
        Get
            'Rückgabe des Werts
            Return _DictPrecedingText
        End Get
        Set(value As String)
            _DictPrecedingText = value
        End Set
    End Property

    ''' <summary>
    ''' Diktat: Schreiben oder Lesen des Werts für das Stoppen des Diktats
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property DictSubsequentText As String
        Get
            'Rückgabe des Werts
            Return _DictSubsequentText
        End Get
        Set(value As String)
            _DictSubsequentText = value
        End Set
    End Property

    ''' <summary>
    ''' Diktat: Senden des Textes an andere Applikationen ein- oder ausschalten
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property DictationSendText As Boolean
        Get
            Return _DictationSendText
        End Get
        Set(value As Boolean)
            _DictationSendText = value
        End Set
    End Property

#End Region



#Region "Speech Recognition Events"

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_AudioLevelUpdated(sender As Object, e As System.Speech.Recognition.AudioLevelUpdatedEventArgs) Handles Engine.AudioLevelUpdated
        'Event auslösen
        RaiseEvent SR_AudioLevelUpdated(sender, e)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_AudioSignalProblemOccurred(sender As Object, e As System.Speech.Recognition.AudioSignalProblemOccurredEventArgs) Handles Engine.AudioSignalProblemOccurred
        'Event auslösen
        RaiseEvent SR_AudioSignalProblemOccurred(sender, e)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_AudioStateChanged(sender As Object, e As System.Speech.Recognition.AudioStateChangedEventArgs) Handles Engine.AudioStateChanged
        'Event auslösen
        RaiseEvent SR_AudioStateChanged(sender, e)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_EmulateRecognizeCompleted(sender As Object, e As System.Speech.Recognition.EmulateRecognizeCompletedEventArgs) Handles Engine.EmulateRecognizeCompleted
        'Event auslösen
        RaiseEvent SR_EmulateRecognizeCompleted(sender, e)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_LoadGrammarCompleted(sender As Object, e As System.Speech.Recognition.LoadGrammarCompletedEventArgs) Handles Engine.LoadGrammarCompleted
        'Event auslösen
        RaiseEvent SR_LoadGrammarCompleted(sender, e)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_RecognizeCompleted(sender As Object, e As System.Speech.Recognition.RecognizeCompletedEventArgs) Handles Engine.RecognizeCompleted
        'Event auslösen
        RaiseEvent SR_RecognizeCompleted(sender, e)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_RecognizerUpdateReached(sender As Object, e As System.Speech.Recognition.RecognizerUpdateReachedEventArgs) Handles Engine.RecognizerUpdateReached
        'Event auslösen
        RaiseEvent SR_RecognizerUpdateReached(sender, e)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SpeechDetected(sender As Object, e As System.Speech.Recognition.SpeechDetectedEventArgs) Handles Engine.SpeechDetected
        'Event auslösen
        RaiseEvent SR_SpeechDetected(sender, e)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SpeechHypothesized(sender As Object, e As System.Speech.Recognition.SpeechHypothesizedEventArgs) Handles Engine.SpeechHypothesized
        'Event auslösen
        RaiseEvent SR_SpeechHypothesized(sender, e)
    End Sub

    ''' <summary>
    ''' Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SpeechRecognitionRejected(sender As Object, e As System.Speech.Recognition.SpeechRecognitionRejectedEventArgs) Handles Engine.SpeechRecognitionRejected
        'Event auslösen
        RaiseEvent SR_SpeechRecognitionRejected(sender, e)
    End Sub

    ''' <summary>
    '''  Speech Recognition Engine Event:
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SpeechRecEngine_SpeechRecognized(sender As Object, e As System.Speech.Recognition.SpeechRecognizedEventArgs) Handles Engine.SpeechRecognized
        'Sound abspielen wenn gewünscht
        If _BeepOn = True Then Media.SystemSounds.Beep.Play()

        'Event auslösen
        RaiseEvent SR_SpeechRecognized(sender, e)

        'Wenn gewünscht, dann den Text auch an eine andere Anwendung senden
        If _DictationSendText Then DictateSendTextToApplications(e.Result.Text)
    End Sub

#End Region



#Region "Timers"

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TimerDecoupleConfidence_Tick(sender As Object, e As EventArgs) Handles TimerDecoupleConfidence.Tick
        'Lokale Variablen
        Dim ThisTimer As Timer = CType(sender, Timer)

        'Den Timer stoppen, da er ja nur einmal aufgerufen werden soll
        ThisTimer.Stop()
        ThisTimer.Enabled = False

        Try
            'Prüfen des Schwellenwerts und gegebenenfalls korrigieren
            If _ConfidenceThreshold > 100 Then _ConfidenceThreshold = 100
            If _ConfidenceThreshold < 0 Then _ConfidenceThreshold = 0

            ''Setzen des Schwellenwerts
            '' Thread beenden
            'If ConfidenceAsyncCall IsNot Nothing Then
            '    With ConfidenceAsyncCall
            '        If .ThreadState = Not Threading.ThreadState.Stopped Then
            '            '250 msec warten ob er sich selbst beendet
            '            .Join(250)
            '            'Sollte der Thread noch laufen, dann hart beenden
            '            If .ThreadState = Not Threading.ThreadState.Stopped Then .Abort()
            '        End If
            '        ConfidenceAsyncCall = Nothing
            '    End With
            'End If
            '' Thread dafür initialisieren
            'ConfidenceAsyncCall = New Threading.Thread(AddressOf ThreadCallConfidence)
            'With ConfidenceAsyncCall
            '    'Thread einstellen/paramietriern
            '    .IsBackground = True
            '    .Priority = Threading.ThreadPriority.BelowNormal
            '    ' Den Thread zum Einstellen der Erkennungs-Sicherheit starten
            '    .Start()
            'End With

            'Speech Recognition Confidence schreiben
            Engine.UpdateRecognizerSetting("CFGConfidenceRejectionThreshold", _ConfidenceThreshold)

            'Speech Recognition Confidence lesen
            Dim MyRecognizerSettingObj As Object = Engine.QueryRecognizerSetting("CFGConfidenceRejectionThreshold")
            'Kann man den Wert einstellen? / Wurde ein Object mit Inhalt zurücgegeben?
            If MyRecognizerSettingObj IsNot Nothing Then
                'Den Wert aus dem Objekt lesen nd speichern
                _ConfidenceThreshold = CType(MyRecognizerSettingObj, Integer)
                'Veränderung bekanntgeben
                RaiseEvent SR_ConfidenceChanged(Engine, New SRConfidenceEventArgs(_ConfidenceThreshold))
                'DEBUGGING
                Debug.Print("Confidence: changed: " & _ConfidenceThreshold.ToString)
            End If

            'Fehler auswerten
        Catch Exc As Exception
            'Fehler an den Aufrufer weiterleiten
            RaiseEvent [Error](Me, New SRErrorEventArgs(Now, "ConfidenceThreshold", Exc))
        End Try
    End Sub

#End Region



    '#Region "Threads"

    '    ''' <summary>
    '    ''' Thread Confidence: Thread initialisieren
    '    ''' </summary>
    '    ''' <remarks></remarks>
    '    Private ConfidenceAsyncCall As Threading.Thread

    '    ''' <summary>
    '    '''  Thread Confidence: Thread Ausführen
    '    ''' </summary>
    '    ''' <remarks></remarks>
    '    Private Sub ThreadCallConfidence()
    '        Try
    '            'Stoppen der Engine wenn diese eingeschaltet ist
    '            If _On Then Engine.RecognizeAsyncStop()

    '            'Speech Recognition Confidence einstellen
    '            Engine.UpdateRecognizerSetting("CFGConfidenceRejectionThreshold", _ConfidenceThreshold)

    '            'Einschalten der Engine, wenn diese vorher eingeschaltet war
    '            If _On Then Engine.RecognizeAsync(RecognizeMode.Multiple)

    '            'Nachricht ausgeben über den neuen Schwellenwert
    '            Call ConfidenceChangedInvoke()

    '        Catch Exc As Threading.ThreadAbortException
    '            Debug.Print("ThreadCallConfidence: aborted")
    '            'Beenden
    '            Exit Sub
    '        End Try
    '    End Sub

    '    ''' <summary>
    '    ''' Thread Confidence: Dem User-Interface informationen übergeben
    '    ''' </summary>
    '    ''' <remarks></remarks>
    '    Private Sub ConfidenceChangedInvoke()
    '        If MyForm.InvokeRequired Then
    '            MyForm.Invoke(New MethodInvoker(AddressOf ConfidenceChangedInvoke))
    '        Else
    '            'Speech Recognition Confidence lesen
    '            Dim ConfidenceLevel As Integer = CInt(Engine.QueryRecognizerSetting("CFGConfidenceRejectionThreshold"))
    '            'Gelesenen Level speichern
    '            _ConfidenceThreshold = ConfidenceLevel
    '            'Veränderung bekanntgeben
    '            RaiseEvent SR_ConfidenceChanged(Engine, New SRConfidenceEventArgs(ConfidenceLevel))
    '            Debug.Print("Confidence: changed")
    '        End If
    '    End Sub

    '#End Region



#Region "Add. Subs and Functions"

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="Engine"></param>
    ''' <param name="OnFlag"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function EngineOnOff(ByRef Engine As SpeechRecognitionEngine, OnFlag As Boolean, Optional Toggle As Boolean = True) As Boolean
        'Lokale Variablen
        Dim RetValue As Boolean = True              'Rückgabewert für Fehlermeldungen / True=Fehlerfrei

        Try
            If OnFlag Then
                'Sind denn schon Grammatiken geladen?
                If Engine.Grammars.Count <= 0 Then
                    'Da keine Grammatik vorhanden ist die Engine wieder
                    ' abschalten
                    OnFlag = False
                    'Fehler an den Aufrufer weiterleiten
                    RaiseEvent [Error](Me, New SRErrorEventArgs(Now, "EngineOnOff", New Exception("No grammars loaded")))
                Else
                    'Erkennung starten, wenn sie gestoppt ist
                    If Engine.AudioState = Recognition.AudioState.Stopped Then
                        Engine.RecognizeAsync(Recognition.RecognizeMode.Multiple)
                    End If
                End If
            Else
                'If Engine.AudioState <> Recognition.AudioState.Stopped Then
                'Erkennung stoppen
                Engine.RecognizeAsyncCancel()
                Engine.RecognizeAsyncStop()
                'End If
            End If

            'Speichern ob eingeschaltet ist
            'intOnFlag = CBool(IIf(Engine.AudioState = Recognition.AudioState.Stopped, False, True))
            _On = OnFlag
            RaiseEvent SR_EngineOn(Me, New SROnEventArgs(_On))

            'Fehler auswerten
        Catch Exc As Exception
            'Fehlermeldung setzen
            RetValue = False
            'Fehler an den Aufrufer weiterleiten
            RaiseEvent [Error](Me, New SRErrorEventArgs(Now, "EngineOnOff", Exc))
        End Try

        'Fehlermeldung oder OK zurückgeben
        Return RetValue
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="SREngine"></param>
    ''' <param name="SRGrammar"></param>
    ''' <param name="FileName"></param>
    ''' <param name="DictationMode"></param>
    ''' <param name="DictationGrammar"></param>
    ''' <param name="DictStart"></param>
    ''' <param name="DictStop"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function SpeechRecLoadGrammar(ByRef SREngine As SpeechRecognitionEngine, ByRef SRGrammar As Grammar, _
                FileName As String, ByRef DictationMode As Boolean, ByRef DictationGrammar As DictationGrammar, _
                DictStart As String, DictStop As String) As String()
        'Lokale Variablen
        Dim RetValue As Boolean = True              'Rückgabewert für Fehlermeldungen / True=Fehlerfrei
        Dim ListOfCommands As String() = Nothing    'Liste der Kommandos 

        Try
            With SREngine
                'Grammatik entladen bzw. entfernen wenn welche geladen wurden
                If .Grammars.Count > 0 Then .UnloadAllGrammars()

                'Ist der Diktat-Modus eingeschaltet?
                If DictationMode Then
                    'Grammatik für Diktat erzeugen
                    If DictationGrammar Is Nothing Then DictationGrammar = New DictationGrammar
                    ' Grammatic für Diktat laden
                    .LoadGrammar(DictationGrammar)
                    'Initialisierung für Diktat stoppen oder starten
                    DictationGrammar.SetDictationContext(DictStart, DictStop)
                Else
                    ' Existiert die Grammatik-Datei überhaupt
                    If Not String.IsNullOrEmpty(FileName) AndAlso IO.File.Exists(FileName) Then
                        'Grammatik initialisieren
                        SRGrammar = New Grammar(FileName, "thema")
                        SRGrammar.Name = "SpeechRecognitionStandard"
                        ' Grammatik asynchron laden
                        .LoadGrammarAsync(SRGrammar)

                        'Worte aus der Grammar anzeigen und als Liste der Befehle speichern
                        Call ReadXMLGrammarToText(FileName, ListOfCommands)
                    End If
                End If
            End With

        Catch Exc As Exception
            'Ein Fehler ist bei der Initialisierung der Spracherkennung aufgetreten
            RetValue = False
            'Fehler an den Aufrufer weiterleiten
            RaiseEvent [Error](Me, New SRErrorEventArgs(Now, "SpeechRecLoadGrammar", Exc))
            'Debuggen
            Debug.Print("Error SpeechRecLoadGrammar: " & Exc.Message)
        End Try

        'Fehlermeldung oder OK zurückgeben
        Return ListOfCommands
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="SREngine"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function SpeechRecEngineInit(ByRef SREngine As SpeechRecognitionEngine, ByRef SRGrammar As Grammar, _
                SRGrammarFileName As String, DictationMode As Boolean, DictationGrammar As DictationGrammar) As Boolean
        'Lokale Variablen
        Dim RetValue As Boolean = True          'Rückgabewert für Fehlermeldungen / True=Fehlerfrei

        Try
            'Sprach-Erkennung initialisieren
            SREngine = New SpeechRecognitionEngine()

            With SREngine
                'Spracherkennung auf das Standard-Audio-Gerät setzen
                .SetInputToDefaultAudioDevice()
            End With

        Catch Exc As Exception
            'Ein Fehler ist bei der Initialisierung der Spracherkennung aufgetreten
            RetValue = False
            'Fehler an den Aufrufer weiterleiten
            RaiseEvent [Error](Me, New SRErrorEventArgs(Now, "SpeechRecEngineInit", Exc))
            'Debuggen
            Debug.Print("Error SpeechRecEngineInit: " & Exc.Message)
        End Try

        'Fehlermeldung oder OK zurückgeben
        Return RetValue
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="DCTimer"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function StdInit(ByRef DCTimer As Timer) As Boolean
        'Lokale Variablen
        Dim RetValue As Boolean = True          'Rückgabewert für Fehlermeldungen / True=Fehlerfrei

        DCTimer = New Timer
        With DCTimer
            .Enabled = False
            .Interval = 850
        End With

        'Fehlermeldung oder OK zurückgeben
        Return RetValue
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="SREngine"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function SpeechRecEngineExit(ByRef SREngine As SpeechRecognitionEngine) As Boolean
        'Lokale Variablen
        Dim RetValue As Boolean = True          'Rückgabewert für Fehlermeldungen / True=Fehlerfrei

        Try                                     'Fehlererkennung starten
            If SREngine IsNot Nothing Then
                With SREngine
                    'Alle Timer stoppen
                    TimerDecoupleConfidence.Enabled = False

                    'Grammatiken entladen, wenn welche vorhanden sind
                    If .Grammars.Count > 0 Then .UnloadGrammar(MyGrammar)
                End With
            End If

        Catch Exc As Exception                   'Fehlerauswertung starten
            'Fehler für die Rückgabe speichern
            RetValue = False
            Debug.Print("SpeechRecEngineExit: " & Exc.Message)
        End Try                                 'Ende der autom. Fehlererkennung

        'Fehlermeldung oder OK zurückgeben
        Return RetValue
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="Filename"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function ReadXMLGrammarToText(Filename As String, ByRef Commands As String()) As Boolean
        'Lokale Variablen
        Dim RetValue As Boolean = True          'Rückgabewert für Fehlermeldungen / True=Fehlerfrei
        Dim CommandStringArray As String()
        Dim iCnt As Integer = 0

        Try
            'Initialisieren der XML-Funktionen
            Dim nsmgr As New XmlNamespaceManager(New NameTable())
            Dim XDoc As New XPathDocument(Filename)
            Dim XNav As XPathNavigator = XDoc.CreateNavigator
            Dim Xiter As XPathNodeIterator
            'Den Namespace-Manager für die XML-Grammar-Datei einstellen
            nsmgr.AddNamespace("default", "http://www.w3.org/2001/06/grammar")
            nsmgr.AddNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance")
            'Navigation auf alle ITEMs setzen -> es wird nur zu ITEM navigiert
            Xiter = XNav.Select("//default:item", nsmgr)

            'Feld in der richtigen größe dimensionieren
            ReDim CommandStringArray(Xiter.Count - 1)
            'Befehle in das Array schreiben
            For iCnt = 0 To Xiter.Count - 1
                Xiter.MoveNext()
                CommandStringArray(iCnt) = Xiter.Current.InnerXml.ToString()
            Next

            'Rückgabe des Strings mit den Kommandos
            Commands = CommandStringArray

        Catch Exc As Exception
            'Ein Fehler ist bei der Initialisierung der Spracherkennung aufgetreten
            RetValue = False
            'Fehler an den Aufrufer weiterleiten
            RaiseEvent [Error](Me, New SRErrorEventArgs(Now, "ReadXMLGrammarToText", Exc))
        End Try

        'Fehlermeldung oder OK zurückgeben
        Return RetValue
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="ChassisNo"></param>
    ''' <param name="Chassis"></param>
    ''' <param name="SpeechRecPossible"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ComputerType(ByRef ChassisNo As Integer, ByRef Chassis As String, ByRef SpeechRecPossible As Boolean) As Boolean
        'Lokale Variablen
        Dim RetValue As Boolean = True          'Rückgabewert für Fehlermeldungen / True=Fehlerfrei
        Dim mos As ManagementObjectSearcher
        Dim moc As ManagementObjectCollection
        Dim mo As ManagementObject = Nothing
        Dim pd As PropertyData
        mos = New Management.ManagementObjectSearcher("SELECT * FROM Win32_SystemEnclosure")
        moc = mos.Get()

        'Vorbelegung des Rückgabewerts
        SpeechRecPossible = False

        Try
            For Each mo In moc
                For Each pd In mo.Properties
                    'Try
                    If Not (pd.Value Is Nothing) Then
                        'Debug.Print("TEST: " & pd.Name & " / " & pd.Value.ToString)
                        Select Case pd.Name
                            Case "ChassisTypes"
                                Dim intChassis() As UInt16 = CType(pd.Value, UInt16())
                                For Each ChassisEntry As UInt16 In intChassis
                                    'Debug.Print("TEST 2: " & ChassisEntry.ToString)
                                    'Nummer des Gehäuses zurückgeben
                                    ChassisNo = ChassisEntry
                                    'Namen des Gehäuses zurückgeben
                                    Select Case ChassisEntry
                                        Case 1
                                            Chassis = "Other"
                                            SpeechRecPossible = False
                                        Case 2
                                            Chassis = "Unknown"
                                            SpeechRecPossible = False
                                        Case 3
                                            Chassis = "Desktop"
                                            SpeechRecPossible = False
                                        Case 4
                                            Chassis = "Low Profile Desktop"
                                            SpeechRecPossible = False
                                        Case 5
                                            Chassis = "Pizza Box"
                                            SpeechRecPossible = False
                                        Case 6
                                            Chassis = "Mini Tower"
                                            SpeechRecPossible = False
                                        Case 7
                                            Chassis = "Tower"
                                            SpeechRecPossible = False
                                        Case 8
                                            Chassis = "Portable"
                                            SpeechRecPossible = True
                                        Case 9
                                            Chassis = "Laptop"
                                            SpeechRecPossible = True
                                        Case 10
                                            Chassis = "Notebook"
                                            SpeechRecPossible = True
                                        Case 11
                                            Chassis = "Handheld"
                                            SpeechRecPossible = True
                                        Case 12
                                            Chassis = "Docking Station"
                                            SpeechRecPossible = False
                                        Case 13
                                            Chassis = "All-in-One"
                                            SpeechRecPossible = True
                                        Case 14
                                            Chassis = "Sub-Notebook"
                                            SpeechRecPossible = True
                                        Case 15
                                            Chassis = "Space Saving"
                                            SpeechRecPossible = False
                                        Case 16
                                            Chassis = "Lunch Box"
                                            SpeechRecPossible = False
                                        Case 17
                                            Chassis = "Main System Chassis"
                                            SpeechRecPossible = False
                                        Case 18
                                            Chassis = "Expansion Chassis"
                                            SpeechRecPossible = False
                                        Case 19
                                            Chassis = "Sub-Chassis"
                                            SpeechRecPossible = False
                                        Case 20
                                            Chassis = "Bus Expansion Chassis"
                                            SpeechRecPossible = False
                                        Case 21
                                            Chassis = "Peripheral Chassis"
                                            SpeechRecPossible = False
                                        Case 22
                                            Chassis = "Storage Chassis"
                                        Case 23
                                            Chassis = "Rack Mount Chassis"
                                            SpeechRecPossible = False
                                        Case 24
                                            Chassis = "Sealed-Case PC"
                                            SpeechRecPossible = False
                                        Case Else
                                            Chassis = "Unknown"
                                            SpeechRecPossible = False
                                    End Select
                                Next
                        End Select
                    End If
                Next
            Next

        Catch
            'Fehlermeldung
            RetValue = False
        Finally
            'Beseitigen der Objekte
            moc.Dispose()
            mos.Dispose()
            mo.Dispose()
        End Try

        'Fehlermeldung zurückgeben 
        Return RetValue
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="DictateText"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function DictateSendTextToApplications(DictateText As String) As Boolean
        'Lokale Variablen
        Dim RetVal As Boolean = True

        Try
            If MyForm.Focus = False Then
                'Daten an die Applikation mit dem Fokus senden
                SendKeys.Send(DictateText)
            End If

            'Fehlererkennung
        Catch Exc As Exception
            'Fehler übergeben
            RetVal = False
            'Debuggen
            Debug.Print("Error DictateSendTextToApplications: " & Exc.Message)
        End Try

        'Rückgabe der Fehlermeldung (Fehler = False)
        Return RetVal
    End Function

#End Region

End Class
